<?php
/* Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use Exception;
use Psr\Log\LoggerInterface;
use InvalidArgumentException;
use App\Constants\ErrorConstants;
use App\Service\ViewTemplateService;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\KernelEvents;
use App\Entity\ViewTemplate\SystemViewTemplate;
use Symfony\Component\HttpKernel\Event\ViewEvent;
use Symfony\Component\HttpFoundation\JsonResponse;
use ApiPlatform\Core\EventListener\EventPriorities;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class ViewTemplateSubscriber implements EventSubscriberInterface
{
    private const UPSERT_SVT = "/api/3/upsert/system_view_templates";

    public function __construct(
        private EntityManagerInterface $entityManager,
        private ViewTemplateService $viewTemplateService,
        private LoggerInterface $logger
    ) {
        $this->entityManager = $entityManager;
        $this->viewTemplateService = $viewTemplateService;
        $this->logger = $logger;
    }

    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::REQUEST => [
                ['handleRequestEvent', EventPriorities::POST_READ],
            ],
            KernelEvents::VIEW => [
                ['handlePostWriteEvent', EventPriorities::POST_WRITE],
            ]
        ];
    }

    public function handleRequestEvent(RequestEvent $event)
    {
        // Fail the put request to mark default=false for the only SVT which is set as default
        $request = $event->getRequest();
        $method = $request->getMethod();
        $path = $request->getPathInfo();
        if (in_array($method, [Request::METHOD_GET, Request::METHOD_DELETE])) {
            return;
        }

        $rawContent = $request->getContent();
        $newData = json_decode($rawContent, true);
        $newIsDefaultValue = is_array($newData) && array_key_exists('isDefault', $newData) ? $newData['isDefault'] : true;

        if ($method == Request::METHOD_PUT) {
            try {
                $previousData = $request->attributes->get('previous_data');

                if ($previousData instanceof SystemViewTemplate) {
                    $oldIsDefaultValue = $previousData->getIsDefault();
                    $this->setResponse($oldIsDefaultValue, $newIsDefaultValue, $event);
                }
            } catch (Exception $ex) {
                $this->logger->error($ex);
            }
        }

        if ($method == Request::METHOD_POST && $path === self::UPSERT_SVT) {
            $uuid = $newData['uuid'];
            $systemViewTemplateRepository = $this->entityManager->getRepository(SystemViewTemplate::class);
            $systemViewTemplate = $systemViewTemplateRepository->findBy(['uuid' => $uuid]);

            if (!empty($systemViewTemplate)) {
                $oldIsDefaultValue = $systemViewTemplate[0]->getIsDefault();
                $this->setResponse($oldIsDefaultValue, $newIsDefaultValue, $event);
            }
        }
    }

    private function setResponse($oldIsDefaultValue, $newIsDefaultValue, $event)
    {
        if ($oldIsDefaultValue && !$newIsDefaultValue) {
            $response = new JsonResponse([
                'message' => ErrorConstants::FSR_CH_0000029
            ], JsonResponse::HTTP_BAD_REQUEST);

            // Set the response to terminate the request
            $event->setResponse($response);
        }
    }

    public function handlePostWriteEvent(ViewEvent $event)
    {
        // If one SVT is marked as default, mark existing default SVT as false
        $request = $event->getRequest();
        $method = $request->getMethod();
        if (in_array($method, [Request::METHOD_GET, Request::METHOD_DELETE])) {
            return;
        }

        $previousData = $request->attributes->get('previous_data');
        $rawContent = $request->getContent();
        $newData = json_decode($rawContent, true);

        if ($previousData instanceof SystemViewTemplate) {
            try {
                $oldIsDefaultValue = $previousData->getIsDefault();
                $newIsDefaultValue = $newData['isDefault'];
                if (!$oldIsDefaultValue && $newIsDefaultValue) {
                    // Among similar system view templates, one should always be marked as default
                    // If one is marked as default=true then existing default should be marked as false
                    $uuid = $previousData->getUuid();
                    $module = $previousData->getModule();
                    $viewOptions = $previousData->getViewOptions();
                    $filters = $previousData->getFilters() ? $previousData->getFilters() : [];
                    $filters = json_encode($filters);

                    $this->unsetDefaultTemplate($module, $viewOptions, $filters, $uuid);
                }
            } catch (Exception $ex) {
                $this->logger->error($ex);
            }
        }

        if (array_key_exists('module', $newData) && array_key_exists('viewOptions', $newData) && $method == Request::METHOD_POST) {
            try {
                $uuid = $newData['uuid'];
                $module = $newData['module'];
                $viewOptions = $newData['viewOptions'];
                $filters = json_encode($newData['filters'] ?? []);
                $isDefault = $newData['isDefault'] ?? false;

                if ($isDefault) {
                    $this->unsetDefaultTemplate($module, $viewOptions, $filters, $uuid);
                }
            } catch (Exception $ex) {
                $this->logger->error($ex);
            }
        }
    }

    private function unsetDefaultTemplate($module, $viewOptions, $filters, $uuid)
    {
        $commandToUpdateSvt = "UPDATE cyops.system_view_template SET isDefault=false WHERE module='$module' AND viewOptions='$viewOptions' AND filters = '$filters' AND uuid NOT IN ('$uuid');";
        $svtEntityDatabase = $this->entityManager->getConnection()->prepare($commandToUpdateSvt);
        try {
            $svtEntityDatabase->execute();
        } catch (InvalidArgumentException | \Exception $e) {
            throw new \Exception('Failure: ' . $e->getMessage());
        }
    }
}
