<?php
/* Copyright start
  Copyright (C) 2008 - 2025 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service\ConfigExportImport;

use Ramsey\Uuid\Uuid;
use Psr\Log\LoggerInterface;
use App\Service\UtilityService;
use App\Constants\AppConstants;
use App\Constants\CacheConstants;
use App\Service\ViewTemplateService;
use Doctrine\ORM\EntityManagerInterface;
use App\Entity\ModelMetadata\ModelMetadata;
use App\Entity\ViewTemplate\SystemViewTemplate;
use Symfony\Component\Serializer\SerializerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

/**
 * Class ViewConfig
 * @package App\Service\ConfigExportImport
 */
class ViewConfig extends PortingConfig
{
    public const NORMALIZER_METHOD = 'jsonld';
    public const SYSTEM_VIEW_TEMPLATE_IRI = "/api/3/system_view_templates/";
    public const NON_MODULE_VIEWS = ['app', 'grid_columns', 'cs_queue_management'];

    public function __construct(
        EntityManagerInterface $entityManager,
        NormalizerInterface $normalizer,
        TokenStorageInterface $tokenStorage,
        ContainerInterface $container,
        protected LoggerInterface $logger,
        protected UtilityService $utilsService,
        protected ViewTemplateService $viewTemplateService,
        protected SerializerInterface $serializer
    ) {
        parent::__construct($entityManager, $normalizer, $tokenStorage, $container, $logger);
        $this->viewTemplateService = $viewTemplateService;
    }

    /**
     * @param array $allData
     * @param string $key
     * @return array
     */
    public function getImportOptions(array $allData, string $typeKey, $tokenType)
    {
        $data = $allData[$typeKey];
        $options = [];
        $repository = $this->loadRepository(SystemViewTemplate::class);
        $navigationItemReplaceOptions = [];
        if (array_key_exists('navigationItemReplaceOptions', $data)) {
            $navigationItemReplaceOptions = $data['navigationItemReplaceOptions'];
            unset($data['navigationItemReplaceOptions']);
        }
        $replaceNavigation = array_key_exists('replaceNavigation', $navigationItemReplaceOptions) &&
            $navigationItemReplaceOptions['replaceNavigation'] ?
            $navigationItemReplaceOptions['replaceNavigation'] : false;
        $navigationItemsToReplace = array_key_exists('navigationItemsToReplace', $navigationItemReplaceOptions) &&
            $navigationItemReplaceOptions['navigationItemsToReplace'] ?
            $navigationItemReplaceOptions['navigationItemsToReplace'] : [];

        foreach ($data as $key => $value) {
            if (!$value) {
                continue;
            }
            if (count(array_filter(array_keys($value), 'is_string')) == 0) {
                // This is a sequential array, used by legacy module view templates
                foreach ($value as $view) {
                    $existing = $repository->findOneBy(['uuid' => $view['uuid']]);
                    $individualOptions = [];
                    $uuid = $view['uuid'];
                    if (!$uuid) {
                        $uuid = implode('-', explode('/', $view['id']));
                    }
                    $individualOptions['uuid'] = $uuid;
                    if ($view['config'] && array_key_exists('title', $view['config'])) {
                        $individualOptions['name'] = $view['config']['title'];
                    }
                    $individualOptions['module'] = $key;
                    $individualOptions['exists'] = $existing ? true : false;
                    $individualOptions['include'] = true;

                    $options[] = $individualOptions;
                }
                continue;
            }

            $existing = null;
            $isExisting = false;
            $individualOptions = [];
            $uuid = array_key_exists('newUuid', $value) ? $value['newUuid'] : $value['uuid']; // Either UUID4 or like 'modules-alerts-detail' or 'app'
            $module = $value['module'] ?? null;
            $viewOptions = $value['viewOptions'] ?? 'detail';
            $filters = $value["filters"] ?? [];
            $isDefault = $value["isDefault"] ?? false;
            $modifyDefaultTemplate = false;

            // Check SVT exists using UUID
            if (Uuid::isValid($uuid)) {
                // Find with UUID (UUID Ex => 'bb4fcb99-1486-4193-a30a-55ea903a0be6')
                // 7.6.2(export) -> 7.6.2(import) FSR provided system view templates(fixtures & sp) will have same uuid on all systems
                $existing = $repository->findOneBy(['uuid' => $uuid]);
                $isExisting = $existing ? true : false;
            } else if (str_starts_with($uuid, 'pages')) {
                $this->logger->info("Old Format Pages identified");
                $value['name'] = $uuid;
                $explodedUuid = explode('-', $uuid);
                $module = $explodedUuid[1];
                $viewOptions = 'detail';
            } else {
                // Old uuid => 'modules-alerts-detail' / 'app'
                if (str_starts_with($uuid, 'modules')) {
                    $this->logger->info("Old Format Module identified");
                    $explodedUuid = explode('-', $uuid);
                    $module = $explodedUuid[1];
                    $viewOptions = $explodedUuid[2];
                    // ToDo: This is also a dirty check
                    $modifyDefaultTemplate = $viewOptions != 'detail';
                } else {
                    $this->logger->info("Old Format app or cs_queue_management identified");
                    $module = $uuid;
                    $modifyDefaultTemplate = true;
                }
            }

            // Handling of system view template name
            $svtName = 'Custom Template';
            $importJobType = $this->importJob ? $this->importJob->getType() : AppConstants::IMPORT_JOB_TYPES["IMPORT_WIZARD"];
            if ($importJobType === 'SolutionPack Import') {
                $svtName = $viewOptions == "settings" ? "Base Setting" : "Base Template";
            }
            $name = array_key_exists('name', $value) ? $value['name'] : $svtName;
            // Check SVT exists using module, view and name
            if (!$isExisting) {
                $this->logger->info("Template not identified by Custom Template name");
                $existing = $repository->findOneBy(['module' => $module, 'viewOptions' => $viewOptions, 'name' => $name]);
                $isExisting = $existing ? true : false;
                $uuid = $existing ? $existing->getUuid() : $uuid;

                // Currently in above scenario we validated, that template exist with name either as
                // Custom Template or Base Template, for an upgraded instance, the older name will be
                // Preserved Template or Old View Template in which case none of the value will match.
                // There are further two scenarios, one with default and one without default.
                // So if in incoming JSON there is no default defined then need to check for Preserved Template.
                // Else get the default template and update the config in same.
                if (!$isExisting && !$isDefault) {
                    $check_preserved_template_name = 'Preserved Template';
                    $existing = $repository->findOneBy(['module' => $module, 'viewOptions' => $viewOptions, 'name' => $check_preserved_template_name]);
                    $isExisting = $existing ? true : false;
                }
                if (!$isExisting && $modifyDefaultTemplate) {
                    // ToDo: Fix this from UI.
                    // This is a dirty check.
                    // Ideally our UI should have shown choosing template option even for list, form and settings
                    $existing = $repository->findOneBy(['module' => $module, 'viewOptions' => $viewOptions, 'isDefault' => true]);
                }
                $isExisting = $existing ? true : false;
                $uuid = $existing ? $existing->getUuid() : $uuid;
                $this->logger->info("UUID: " . $uuid);
            }

            if ($value['config'] && array_key_exists('title', $value['config'])) {
                $individualOptions['name'] = $value['config']['title'];
            }

            if ($module == 'app') {
                if ($existing) {
                    foreach ($value['config']['navigation'] as $key =>  $item) {
                        foreach ($existing->getConfig()['navigation'] as $existingItem) {
                            if ($existingItem['title'] == $item['title']) {
                                $value['config']['navigation'][$key]['exists'] = true;
                                $value['config']['navigation'][$key]['include'] = true;
                                $itemMergeType = in_array($item['title'], $navigationItemsToReplace) ? AppConstants::MERGE_TYPES['replace'] : AppConstants::MERGE_TYPES['merge'];
                                $value['config']['navigation'][$key]['mergeType'] = $itemMergeType;
                            }
                        }
                    }
                }
                foreach ($value['config']['navigation'] as $key =>  $item) {
                    if (!array_key_exists("exists", $value['config']['navigation'][$key]) || (array_key_exists("exists", $value['config']['navigation'][$key]) && $value['config']['navigation'][$key]['exists'] == false)) {
                        $value['config']['navigation'][$key]['exists'] = false;
                        $value['config']['navigation'][$key]['include'] = true;
                        $value['config']['navigation'][$key]['mergeType'] = AppConstants::MERGE_TYPES['merge'];
                    }
                }
                $replaceEntireNavigation = $replaceNavigation && count($navigationItemsToReplace) == 0 ? true : false;
                $individualOptions['replaceEntireNavigation'] = $replaceEntireNavigation;

                // If all navigation items are intended to replace then global mergeType will be replace
                // If all navigation items are intended to append then global mergeType will be append
                // If some navigation items are having mergeType as replace and some having append then global mergeType will be empty string
                $individualOptions['mergeType'] = $replaceNavigation ? (count($navigationItemsToReplace) == 0 ? AppConstants::MERGE_TYPES['replace'] : "") : AppConstants::MERGE_TYPES['merge'];
            }
            if ($viewOptions && $viewOptions == 'settings') {
                $individualOptions['_hidden'] = true;
            }

            // Handling of 'system' property of view template
            if ($importJobType === AppConstants::IMPORT_JOB_TYPES["IMPORT_WIZARD"] && $existing) {
                // Keep existing value of 'system' when importing through import wizard
                $individualOptions['system'] = $existing->getSystem();
            } elseif ($importJobType === AppConstants::IMPORT_JOB_TYPES["SOLUTION_PACK_IMPORT"]) {
                // Add 'system = true' when importing through solution pack
                $individualOptions['system'] = true;
            } else {
                // Add value of 'system' present in config, if not then add 'system = false'
                $individualOptions['system'] = $value["system"] ?? false;
            }

            // Update the existing
            $individualOptions['uuid'] = Uuid::isValid($uuid) ? $uuid : Uuid::uuid4()->toString();
            $individualOptions['exists'] = $isExisting ? true : false;
            $individualOptions['name'] = $existing ? $existing->getName() : $name;
            $individualOptions['module'] = $module;
            $individualOptions['viewOptions'] = $viewOptions;
            $individualOptions['filters'] = $filters;
            $individualOptions['include'] = true;
            $individualOptions['isDefault'] = $existing ? $existing->getIsDefault() : $isDefault;
            $individualOptions['importedBy'] = $existing && property_exists($existing, 'importedBy') ? $existing->getImportedBy() : [];
            $individualOptions['config'] = $value['config'];
            $options[] = $individualOptions;
        }
        return $options;
    }

    /**
     * Finds the value in the array by key
     *
     * @param array $array
     * @param string $key
     * @param string $value
     * @return mixed
     */
    protected static function findSvtBy(array $array, $keyValueArray, $caseInsensitive = false)
    {
        $requiredCount = count($keyValueArray);
        foreach ($array as $item) {
            $matchCount = 0;
            foreach ($keyValueArray as $key => $value) {
                if ((!$caseInsensitive && $item[$key] == $value) || ($caseInsensitive && strtolower($item[$key]) == strtolower($value))) {
                    $matchCount++;
                }
            }
            if ($requiredCount == $matchCount) {
                return $item;
            }
        }
        return null;
    }

    /**
     * Imports the config data using the specified options
     *
     * @param array $data
     * @param array $options
     * @return mixed
     */
    public function import(array $data, array $options, $tokenType, $importedBy = [])
    {
        $toAdd = [];
        $toUpdate = [];
        $defaultSvtArray = [];
        $importedSvtUuidArray = [];

        // This iteration will check for default svt present in imported config
        // If default svt for a module view is not present in system then 'isDefault=True' value from config will be honoured
        foreach ($data as $value) {
            if (is_array($value) && array_key_exists('isDefault', $value) && $value['isDefault']) {
                $this->logger->info("Request consists of making change to default template");
                $viewTemplate = $this->viewTemplateService->getViewTemplateData(
                    SystemViewTemplate::class,
                    $value['module'],
                    $value['viewOptions'],
                    $value['filters']
                );
                $defaultViewTemplate = is_array($viewTemplate) && !empty($viewTemplate) ? $viewTemplate[0] : null;
                $defaultSvt = json_encode(['module' => $value['module'], 'viewOptions' => $value['viewOptions'], 'filters' => $value['filters']]);
                if (!$defaultViewTemplate && !in_array($defaultSvt, $defaultSvtArray)) {
                    $defaultSvtArray[] = $defaultSvt;
                }
            } else {
                $this->logger->info("Request does not consists of making change to default template");
            }
        }

        foreach ($options as $option) {
            $this->logger->info("View Option " . $option['viewOptions']);
            if (is_array($option) && array_key_exists("_hidden", $option) && $option['_hidden'] && $option['viewOptions'] == 'settings') {
                $keyValueArray = ['module' => $option['module'], 'viewOptions' => 'detail'];
                $detailView = $this->findSvtBy($options, $keyValueArray);
                $option['include'] = $detailView['include'];
            }
            if (!$option['include']) {
                $this->logger->info("Skipping the view template");
                // Do not merge skipped items
                continue;
            }
            # When imported through import wizard, 'mergeType' value selected in wizard is honoured
            $option['appendNavigation'] = array_key_exists('mergeType', $option) && ($option['mergeType'] == AppConstants::MERGE_TYPES['replace'] ||
                $option['mergeType'] == '') ? false : true;

            $foundView = null;
            foreach ($data as $value) {
                if (!$value) {
                    continue;
                }
                if (count(array_filter(array_keys($value), 'is_string')) == 0) {
                    // This is a sequential array, used by module view templates
                    foreach ($value as $view) {
                        if ($option['uuid'] == $view['uuid']) {
                            $foundView = $view;
                            break;
                        }
                    }
                    continue;
                }

                $uuid = array_key_exists('newUuid', $value) ? $value['newUuid'] : $value['uuid'];
                $module = $value['module'] ?? null;
                $viewOptions = $value['viewOptions'] ?? 'detail';
                $filters = $value["filters"] ?? [];

                if (!Uuid::isValid($uuid)) {
                    $this->logger->info("Not a valid uuid case");
                    if (str_starts_with($uuid, 'pages')) {
                        $this->logger->info("Not a valid uuid case: Condition Page");
                        $value['name'] = $uuid;
                        $explodedUuid = explode('-', $uuid);
                        $module = $explodedUuid[1];
                        $viewOptions = 'detail';
                    }
                    // Old uuid => 'modules-alerts-detail' / 'app'
                    else if (str_starts_with($uuid, 'modules')) {
                        $this->logger->info("Not a valid uuid case: Condition Module");
                        $explodedUuid = explode('-', $uuid);
                        $module = $explodedUuid[1];
                        $viewOptions = $explodedUuid[2];
                    } else {
                        $this->logger->info("Not a valid uuid case: Condition Default");
                        $module = $uuid;
                    }
                }

                $value['module'] = $module;
                $value['viewOptions'] = $viewOptions;
                $value['filters'] = $filters;

                $this->logger->info("** Module **");
                $this->logger->info("Module " . $module);
                $this->logger->info("View Options " . $viewOptions);

                $importJobType = $this->importJob ? $this->importJob->getType() : AppConstants::IMPORT_JOB_TYPES["IMPORT_WIZARD"];
                $productType = $this->utilsService->getRequiredLicenseDetails(CacheConstants::PRODUCT_TYPE);
                if ($value['module'] == "app"  && $option['module'] == $value['module'] && !in_array($uuid, $importedSvtUuidArray)) {
                    if ($productType == AppConstants::PRODUCT_TYPE['fortitip'] && $importJobType == 'SolutionPack Import' && !$this->containsFortiTIP($importedBy))
                        continue;
                    $importedSvtUuidArray[] = $uuid;
                    $value['uuid'] = $option['uuid'];
                    $this->handleIsDefaultValue($defaultSvtArray, $value);
                    $viewTemplate = $this->viewTemplateService->getViewTemplateData(SystemViewTemplate::class, $module, $viewOptions, $filters);
                    $defaultViewTemplate = $viewTemplate[0];
                    $normalizedData = $this->serializer->normalize($defaultViewTemplate, self::NORMALIZER_METHOD, ['resource_class' => SystemViewTemplate::class]);
                    $oldNav = $normalizedData;
                    if ($option['appendNavigation']) {
                        $foundView = $this->mergeNAppendNav($option['config']['navigation'], $oldNav);
                        $foundView['name'] = $option['name'];
                        $foundView['system'] = $option['system'];
                        break;
                    } else {
                        $foundView = $this->replaceNavItems($option, $oldNav);
                        $foundView['name'] = $option['name'];
                        $foundView['system'] = $option['system'];
                        break;
                    }
                } elseif (
                    $option['module'] == $value['module'] && $option['viewOptions'] == $value['viewOptions']
                    && $option['filters'] == $value['filters'] && !in_array($uuid, $importedSvtUuidArray)
                ) {
                    $importedByArray = $option['importedBy'];
                    if ($importedByArray) {
                        foreach ($option['importedBy'] as $importedByValue) {
                            if ($importedBy && $importedBy[0]['apiName'] != $importedByValue['apiName']) {
                                $importedByArray = array_merge($importedByArray, $importedBy);
                            }
                        }
                    } else {
                        $importedByArray = $importedBy;
                    }

                    $value['importedBy'] = $importedByArray;
                    $importedSvtUuidArray[] = $uuid;
                    $value['uuid'] = $option['uuid'];
                    $value['name'] = $option['name'];
                    $value['system'] = $option['system'];
                    $this->handleIsDefaultValue($defaultSvtArray, $value);

                    $foundView = $value;
                    break;
                }
            }
            if ($option['exists'] && $foundView) {
                $toUpdate[] = $foundView;
            } elseif ($foundView) {
                $toAdd[] = $foundView;
            }
        }
        $results = [];
        if (!empty($toAdd)) {
            $fullURI = sprintf('%s/api/3/', $this->container->getParameter(AppConstants::CRUDHUB_PARAMETER_KEY));
            $result = $this->sendRequest($fullURI, 'post', $toAdd, $tokenType);
            $results[] = $result;
        }
        if (!empty($toUpdate)) {
            $fullURI = sprintf('%s/api/3/update/system_view_templates', $this->container->getParameter(AppConstants::CRUDHUB_PARAMETER_KEY));
            $body = [
                "data" => $toUpdate
            ];
            $result = $this->sendRequest($fullURI, 'put', $body, $tokenType);
            $results[] = $result;
        }

        if (in_array(false, $results)) {
            return false;
        }
        return true;
    }

    private function handleIsDefaultValue(&$defaultSvtArray, &$svt)
    {
        $viewTemplate = $this->viewTemplateService->getViewTemplateData(SystemViewTemplate::class, $svt['module'], $svt['viewOptions'], $svt['filters']);
        $defaultViewTemplate = is_array($viewTemplate) && !empty($viewTemplate) ? $viewTemplate[0] : null;
        $defaultSvt = json_encode(['module' => $svt['module'], 'viewOptions' => $svt['viewOptions'], 'filters' => $svt['filters']]);

        if (!$defaultViewTemplate && !in_array($defaultSvt, $defaultSvtArray)) {
            $svt['isDefault'] = true;
        } elseif ($defaultViewTemplate && $defaultViewTemplate->getUuid() != $svt['uuid']) {
            $svt['isDefault'] = false;
        } elseif ($defaultViewTemplate && $defaultViewTemplate->getUuid() == $svt['uuid']) {
            $svt['isDefault'] = true;
        }
        $defaultSvtArray[] = $defaultSvt;
    }

    /**
     * Exports the config using the specified options
     *
     * @param array $options
     * @return array
     */
    public function export(array $options, $tokenType = 'JWT')
    {
        return $this->getResourceFromIRI($options, SystemViewTemplate::class);
    }

    /**
     * Exports the config using the specified options
     *
     * @param array $options
     * @return array
     */
    public function exportFile(array $options, $type, $tmpDirectory, $tokenType = 'JWT')
    {
        $modifiedOptions = [];
        $appendNavigation = [];
        $navigationOptions = [];
        $navigationTitleMergeTypeMap = [];
        $metaData = [];
        $key = null;
        foreach ($options as $value) {
            if (is_string($value)) {
                $modifiedOptions[] = $value;
            }
            if (is_array($value) && !empty($value)) {
                $modifiedOptions[] = is_string($value['uuid']) && str_contains($value['uuid'], "/") ? $value['uuid'] : self::SYSTEM_VIEW_TEMPLATE_IRI . $value['uuid'];
                if (array_key_exists('value', $value) && $value['value'] == "app") {
                    $appendNavigation = array_key_exists('appendNavigation', $value) && !empty($value['appendNavigation']) ? $value['appendNavigation'] : [];
                    $navigationOptions = array_key_exists('navigationOptions', $value) && !empty($value['navigationOptions']) ? $value['navigationOptions'] : [];
                    $replaceEntireNavigation = array_key_exists('mergeType', $value) && $value['mergeType'] == AppConstants::MERGE_TYPES['replace'] ? true : false;
                }
            }
        }
        $data = $this->export($modifiedOptions, $tokenType);
        if (!$data) {
            return $data;
        }
        if ($navigationOptions) {
            foreach ($navigationOptions as $navigationOption) {
                $navigationTitleMergeTypeMap[$navigationOption['title']] = $navigationOption['mergeType'];
            }
        }
        foreach ($data as $item) {
            $module = $item['module'] ?? "";
            $viewOptions = $item['viewOptions'] ?? "";
            $filters = $item['filters'] ?? "";
            $isDefault = $item['isDefault'] ?? false;
            if ($viewOptions && !in_array($module, self::NON_MODULE_VIEWS)) {
                if (($filters == null || $filters == []) && $isDefault) {
                    if ($viewOptions === 'settings') {
                        $typeName = 'settings.json';
                    } else {
                        $typeName = $viewOptions . '-layout.json';
                    }
                    $item['newUuid'] = $item['uuid'];
                    $item['uuid'] = 'modules-' . $module . '-' . $viewOptions;
                } else {
                    $uniqueNum = explode('-', $item['uuid'])[1];
                    if ($viewOptions === 'settings') {
                        $typeName = $item['name'] . '-' . $uniqueNum . '-settings.json';
                    } else {
                        $typeName = $item['name'] . '-' . $uniqueNum . '-' . $viewOptions . '-layout.json';
                    }
                }
                $filename = $tmpDirectory . 'modules/' . $this->cleanFileName($module) . '/' . $typeName;
                $metaData[$module][] = $viewOptions;
            } else {
                if ($module == 'app') {
                    $navigationItemsToReplace = [];
                    $module = 'navigation';
                    $metaData[$module] = [];
                    $appendList = [];
                    $navigation = [];
                    foreach ($item['config']['navigation'] as $value) {
                        $navigationTitle = $value['title'];
                        if (in_array($navigationTitle, $appendNavigation)) {
                            $value = $this->sortArrayByKey($value); // First and second level sorting
                            foreach ($value as $key => $arrayElement) {
                                if (is_array($arrayElement) && !empty($arrayElement)) {
                                    $value[$key] = $this->sortArrayByKey($value[$key]); // Third level sorting
                                    foreach ($arrayElement as $arrayKey => $arrayValue) {
                                        if (is_array($arrayValue) && !empty($arrayValue)) {
                                            $arrayElement[$arrayKey] = $this->sortArrayByKey($arrayElement[$arrayKey]); // Fourth level sorting
                                        }
                                    }
                                }

                                $value[$key] = $arrayElement;
                            }
                            $appendList[] = $value;
                            array_push($navigation, $navigationTitle);
                            if (
                                $navigationTitleMergeTypeMap && array_key_exists($navigationTitle, $navigationTitleMergeTypeMap) &&
                                $navigationTitleMergeTypeMap[$navigationTitle] == AppConstants::MERGE_TYPES['replace']
                            ) {
                                array_push($navigationItemsToReplace, $navigationTitle);
                            }
                        }
                    }
                    $metaData[$module] = $navigation;
                    $item['config']['navigation'] =  $appendList;
                    $navigationItemReplaceOptionsFile = $tmpDirectory . 'views/' . AppConstants::NAVIGATION_OPTION_FOLDER . '/' . AppConstants::NAVIGATION_ITEM_REPLACE_OPTIONS_JSON;
                    $navigationItemReplaceOptions['replaceNavigation'] = $replaceEntireNavigation ? true : (count($navigationItemsToReplace) > 0 ? true : false);
                    $navigationItemReplaceOptions['navigationItemsToReplace'] = $navigationItemsToReplace;
                    $this->writeToFile($navigationItemReplaceOptionsFile, $navigationItemReplaceOptions);
                }
                if ($filters == null || $filters == []) {
                    $item['newUuid'] = $item['uuid'];
                    $item['uuid'] = $module;
                }
                $filename = $tmpDirectory . 'views/' . $module . '.json';
            }
            $this->writeToFile($filename, $item);
            $filenames[] = $filename;
        }
        if ($type == 'viewTemplates') {
            $viewMetaData = [];
            foreach ($metaData as $key => $value) {
                $mmd = $this->getMetadataByType($key);
                $views = array_values(array_unique($value));
                $viewMetaData[$key] = ['name' => $mmd->getDescriptions()['plural'], 'apiName' => $key, 'views' => $views];
            }
            $metaData = $viewMetaData;
        }
        $infoJsonFile = $tmpDirectory . 'info.json';
        if (file_exists($infoJsonFile)) {
            // If only views are exported through SP, then info.json will not contain 'modules' under 'contents'
            // While uploading SP, viewtemplates will not be imported as folders are removed which are not mentioned under 'contents' in info.json
            $infoJson = file_get_contents($infoJsonFile);
            $infoJsonData = json_decode($infoJson, true);
            if (array_key_exists('contents', $infoJsonData) and !array_key_exists('modules', $infoJsonData['contents'])) {
                $this->appendToFile($infoJsonFile, 'modules', []);
            }
        }
        $this->appendToFile($tmpDirectory . 'info.json', $type, $metaData);
        return $filenames;
    }

    /**
     * Generates import options from the tmp folder
     *
     * @param string $key
     * @param string $tmpDirectory
     * @param string $tokenType
     * @return array
     */
    public function getImportOptionsFromFile(string $key, string $tmpDirectory, $tokenType)
    {
        $data = $this->loadDataFromDirectory($key, $tmpDirectory);
        return $this->getImportOptions([$key => $data], $key, $tokenType);
    }

    /**
     * Imports from the tmp folder
     *
     * @param array $options
     * @param string $key
     * @param string $tmpDirectory
     * @param [type] $tokenType
     * @return void
     */
    public function importFromFile(array $options, string $key, string $tmpDirectory, $tokenType, $importedBy = [])
    {
        $data = $this->loadDataFromDirectory($key, $tmpDirectory);
        return $this->import($data, $options, $tokenType, $importedBy);
    }

    private function getViewTemplateData(string $folderPath): array
    {
        $data = [];
        if (file_exists($folderPath)) {
            $files = scandir($folderPath);
        }
        foreach ($files as $file) {
            $filePath = $folderPath . "/" . $file;
            if (file_exists($filePath)) {
                if (str_ends_with($file, '-layout.json') || $file === 'settings.json' || str_ends_with($file, '-settings.json')) {
                    $fileData = file_get_contents($filePath);
                    $jsonData = json_decode($fileData, true);
                    $data[] = $jsonData;
                }
            }
        }
        return $data;
    }

    protected function loadDataFromDirectory(string $key, string $tmpDirectory)
    {
        // Module views
        if ($key == 'viewTemplates') {
            $folderPath = $tmpDirectory . 'modules/';
            $folders = [];
            if (file_exists($folderPath)) {
                $folders = scandir($folderPath);
            }
            $data = [];
            foreach ($folders as $folder) {
                $folderData = $this->getViewTemplateData($folderPath . $folder);
                $data = array_merge($data, $folderData);
            }
        } else {
            // Other views
            $folderPath = $tmpDirectory . $key . '/';
            $folders = [];
            if (file_exists($folderPath)) {
                $folders = scandir($folderPath);
            }
            $data = [];
            foreach ($folders as $file) {
                $viewPath = $folderPath . $file;
                if (file_exists($viewPath)) {
                    if (is_dir($viewPath) && $file == AppConstants::NAVIGATION_OPTION_FOLDER) {
                        $viewPath = $viewPath . '/' . AppConstants::NAVIGATION_ITEM_REPLACE_OPTIONS_JSON;
                        $file = AppConstants::NAVIGATION_ITEM_REPLACE_OPTIONS_JSON;
                    }
                    $fileData = file_get_contents($viewPath);
                    $jsonData = json_decode($fileData, true);
                    if (isset($jsonData['uuid']) && $jsonData['uuid'] == 'navigation') {
                        $jsonData['uuid'] = 'app';
                    }
                    if ($file == AppConstants::NAVIGATION_ITEM_REPLACE_OPTIONS_JSON) {
                        $data['navigationItemReplaceOptions'] = $jsonData;
                    } else {
                        $data[] = $jsonData;
                    }
                }
            }
        }

        return $data;
    }

    public function generateExportOptions(array $options, $tokenType = 'JWT')
    {
        // $options are from ImportJob
        $exportOption = [];
        $navigationOptions = [];
        $navigationItemReplaceOptions = [];
        $fileExtractedPath = $this->tmpFilePath;
        $navigationOptionsJsonPath = $fileExtractedPath . 'views/' . AppConstants::NAVIGATION_OPTION_FOLDER . '/' . AppConstants::NAVIGATION_ITEM_REPLACE_OPTIONS_JSON;
        if (file_exists($navigationOptionsJsonPath)) {
            $jsonData = file_get_contents($navigationOptionsJsonPath);
            $navigationItemReplaceOptions = json_decode($jsonData, true);
        }
        foreach ($options as $value) {
            if ($value['module'] == 'app') {
                $value['appendNavigation'] = [];
                $navigationItemsToReplace = $navigationItemReplaceOptions && array_key_exists('navigationItemsToReplace', $navigationItemReplaceOptions) ? $navigationItemReplaceOptions['navigationItemsToReplace'] : [];
                $navMergeType = count($navigationItemsToReplace) == 0 ? AppConstants::MERGE_TYPES['merge'] : (count($value['appendNavigation']) > count($navigationItemsToReplace) ? "" : AppConstants::MERGE_TYPES['replace']);

                foreach ($value['config']['navigation'] as $item) {
                    $value['appendNavigation'][] = $item['title'];
                    $mergeType = in_array($item['title'], $navigationItemsToReplace) ? AppConstants::MERGE_TYPES['replace'] : AppConstants::MERGE_TYPES['merge'];
                    $navigationOptions[] = ['title' => $item['title'], 'mergeType' => $mergeType];
                }
                array_push($exportOption, ['uuid' => $value['uuid'], 'value' => $value['module'], 'mergeType' => $navMergeType, 'appendNavigation' => $value['appendNavigation'], 'navigationOptions' => $navigationOptions]);
            } else {
                $viewData = ['uuid' => $value['uuid'], 'module' => $value['module'], 'filters' => $value['filters'], 'viewOptions' => $value['viewOptions']];
                array_push($exportOption, $viewData);
            }
        }
        return $exportOption;
    }

    public function generateInfoContent(array $options)
    {
        // $options are from exportTemplate
        $content = [];
        foreach ($options as $item) {
            if (is_array($item)) {
                if (array_key_exists("value", $item) && $item["value"] == "app") {
                    $content["navigation"] = $item["appendNavigation"];
                } elseif (array_key_exists("value", $item) && in_array($item["value"], ['cs_queue_management', 'grid_columns'])) {
                    continue;
                } else {
                    $moduleType = $item['module'];
                    $viewOptions = $item['viewOptions'];
                    if (!array_key_exists($moduleType, $content)) {
                        $module = $this->getMetadataByType($moduleType);
                        if (is_null($module)) {
                            // For view template which are not associated with any module, like cs_queue_management etc are failing because of this
                            $this->logger->info("No module exists with name " . $moduleType);
                            continue;
                        }
                        $content[$moduleType] = [
                            "name" => $module->getDescriptions()['plural'],
                            "apiName" => $moduleType,
                            "views" => [$viewOptions]
                        ];
                    } else {
                        array_push($content[$moduleType]["views"], $viewOptions);
                    }
                    sort($content[$moduleType]["views"]);
                }
            }
        }
        return $content;
    }

    public function getMetadataByType($type)
    {
        $repository = $this->entityManager->getRepository(ModelMetadata::class);
        return $repository->findOneBy(['type' => $type]);
    }

    protected function mergeNAppendSubNav($oldSubNavItems, $newSubNavItems)
    {
        $updatedSubNavItems = $oldSubNavItems;
        foreach ($newSubNavItems as $newItem) {
            $itemExists = false;
            foreach ($oldSubNavItems as $key => $oldItem) {
                if ($oldItem['title'] == $newItem['title']) {
                    $itemExists = true;
                    $updatedSubNavItems[$key] = $newItem;
                    break;
                }
            }
            if (!$itemExists) {
                array_push($updatedSubNavItems, $newItem);
            }
        }

        return $updatedSubNavItems;
    }

    protected function mergeNAppendNav($newNav, $oldNav)
    {
        $updatedNav = $oldNav;
        foreach ($newNav as $key => $newNavItem) {
            $include = (array_key_exists("include", $newNavItem) && $newNavItem['include'] == true) || (array_key_exists("include", $newNavItem) && $newNavItem['include'] == "true") || (array_key_exists('mergeType', $newNavItem) && $newNavItem['mergeType'] != AppConstants::MERGE_TYPES['skip']);
            if ($include) {
                $itemExists = false;
                unset($newNavItem['include']);
                unset($newNavItem['exists']);
                foreach ($oldNav['config']['navigation'] as $key => $oldNavItem) {
                    $updatedNavItems = [];
                    if ($oldNavItem['title'] == $newNavItem['title']) {
                        $itemExists = true;
                        $oldNavItems = $oldNavItem['items'] ?? [];
                        $newNavItems = $newNavItem['items'] ?? [];

                        $updatedNav['config']['navigation'][$key] = $newNavItem; //merge values like icon from the new nav
                        // $updatedNav['config']['navigation'][$key]['items'] =  $oldNavItems; //keep the items from old nav, as we would be matching each sub item in seoerate for loop
                        if (empty($oldNavItems) && !empty($newNavItems)) {
                            $updatedNavItems = $newNavItems;
                        }

                        if (!empty($oldNavItems) && !empty($newNavItems)) {
                            $updatedNavItems = $this->mergeNAppendSubNav($oldNavItems, $newNavItems);
                        }
                        if (!empty($updatedNavItems)) {
                            $updatedNav['config']['navigation'][$key]['items'] = $updatedNavItems; //?? incase both old and new item is empty
                        }
                    }
                }
                if (!$itemExists) {
                    array_push($updatedNav['config']['navigation'], $newNavItem);
                }
            }
        }
        return $updatedNav;
    }

    protected function replaceNavItems($newNav, $oldNav)
    {
        $updatedNav = $oldNav;
        # Replace entire navigation
        if ($newNav['replaceEntireNavigation']) {
            return $newNav;
        }
        # Handle replace and append of navigation items as per individual options
        foreach ($newNav['config']['navigation'] as $newNavItemkey => $newNavItem) {
            if ((array_key_exists("include", $newNavItem) && $newNavItem['include'] == true) || (array_key_exists('mergeType', $newNavItem) && $newNavItem['mergeType'] != AppConstants::MERGE_TYPES['skip'])) {
                $itemExists = false;
                foreach ($oldNav['config']['navigation'] as $oldNavItemkey => $oldNavItem) {
                    if ($newNavItem['title'] == $oldNavItem['title']) {
                        $itemExists = true;
                        if (array_key_exists('mergeType', $newNavItem) && $newNavItem['mergeType'] == AppConstants::MERGE_TYPES['replace']) {
                            $updatedNav['config']['navigation'][$oldNavItemkey] = $newNavItem;
                        } else {
                            $oldNavItems = $oldNavItem['items'] ?? [];
                            $newNavItems = $newNavItem['items'] ?? [];
                            $updatedNavItems = $this->mergeNAppendSubNav($oldNavItems, $newNavItems);

                            if (!empty($updatedNavItems)) {
                                $updatedNav['config']['navigation'][$oldNavItemkey]['items'] = $updatedNavItems; //?? incase both old and new item is empty
                            }
                        }
                    }
                }
                if (!$itemExists) {
                    array_push($updatedNav['config']['navigation'], $newNavItem);
                }
            }
        }
        return $updatedNav;
    }

    // Function to check if 'Forti-TIP' exists in 'apiName'
    protected function containsFortiTIP($array)
    {
        foreach ($array as $element) {
            if (isset($element['apiName']) && $element['apiName'] === 'fortiTIP') {
                return true; // Return true if 'Forti TIP' is found
            }
        }
        return false; // Return false otherwise
    }
}
