<?php
/* Copyright start
  Copyright (C) 2008 - 2025 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service;

use Doctrine\ORM\EntityManager;
use ApiPlatform\Api\IriConverterInterface;
use App\Entity\ViewTemplate\UserViewTemplate;
use App\Entity\ViewTemplate\SystemViewTemplate;
use Symfony\Component\Serializer\SerializerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class ViewTemplateService
{
    /** @var TokenStorageInterface */
    protected $tokenStorage;

    /** @var IriConverterInterface */
    protected $iriConverter;

    /** @var EntityManager */
    protected $entityManager;

    /** @var SerializerInterface */
    protected $serializer;

    protected $containers;

    public function __construct(
        TokenStorageInterface $tokenStorage,
        IriConverterInterface $iriConverter,
        ContainerInterface $containers
    ) {
        $this->tokenStorage = $tokenStorage;
        $this->iriConverter = $iriConverter;
        $this->containers = $containers;
        $this->entityManager = $this->containers->get('doctrine.orm.entity_manager');
    }

    /**
     * @return string
     */
    private function getCurrentUserAsIri()
    {
        $user = $this->tokenStorage->getToken()->getUser();
        return $this->iriConverter->getIriFromResource($user);
    }

    /**
     * @param $name
     *
     * @return SystemViewTemplate|UserViewTemplate
     */
    public function getViewTemplate($module, $viewOptions, $filters)
    {
        // First check in UVT
        $viewTemplate = $this->getViewTemplateData(UserViewTemplate::class, $module, $viewOptions, $filters);

        if (!count($viewTemplate) || is_null($viewTemplate)) {
            // If not found in UVT, check in SVT
            $viewTemplate = $this->getViewTemplateData(SystemViewTemplate::class, $module, $viewOptions, $filters);
        }

        if (count($viewTemplate)) {
            $viewTemplate = $viewTemplate[0];
        } else {
            throw new NotFoundHttpException();
        }

        return $viewTemplate;
    }

    /**
     * @param UserViewTemplate $viewTemplate
     */
    public function saveViewTemplate($name, $viewTemplate)
    {
        $viewTemplate->setName($name);
        $viewTemplate->setOwner($this->getCurrentUserAsIri());

        $this->entityManager->persist($viewTemplate);
        $this->entityManager->flush();
    }

    /**
     * @param SystemViewTemplate $viewTemplate
     */
    public function saveSystemViewTemplate($viewTemplate)
    {
        $this->entityManager->persist($viewTemplate);
        $this->entityManager->flush();
    }

    public function getUserViewTemplate($entityRepository, $user, $name)
    {
        return $entityRepository->findBy(["owner" => $user, "name" => $name]);
    }

    public function getViewTemplateData($class, $module = null, $viewOptions = null, $filters = [])
    {
        $user = $this->getCurrentUserAsIri();
        $qb = $this->entityManager->createQueryBuilder();

        $qb->select('vt')
            ->from($class, 'vt');

        if ($module) {
            $qb->andWhere('vt.module = :module')
                ->setParameter('module', $module);
        }

        if ($viewOptions) {
            $qb->andWhere('vt.viewOptions = :viewOptions')
                ->setParameter('viewOptions', $viewOptions);
        }

        // One of the view template will always be default
        $qb->andWhere('vt.isDefault = :isDefault')
            ->setParameter('isDefault', true);

        $queryBuilder = clone $qb;

        // if ($filters) {
        //     foreach ($filters as $field => $value) {
        //         $qb->andWhere("JSONB_EXTRACT_PATH_TEXT(vt.filters, :field_$field) = :value_$field")
        //             ->setParameter("field_$field", $field)
        //             ->setParameter("value_$field", $value);
        //     }
        // }

        if ($class == UserViewTemplate::class) {
            $qb->andWhere('vt.owner = :user')
                ->setParameter('user', $user);
        }

        $vtResult = $qb->getQuery()->getResult();

        if ($filters) {
            ksort($filters);
        } else {
            $filters = [];
        }

        $result = [];
        foreach ($vtResult as $vt) {
            $vtFilters = $vt->getFilters() ? $vt->getFilters() : [];
            ksort($vtFilters);

            if ($filters === $vtFilters) {
                $result[] = $vt;
                break;
            }
        }

        if ($class == SystemViewTemplate::class && count($result) == 0) {
            // If system view template with exact filter is not found, fallback is default svt
            $svtResult = $queryBuilder->getQuery()->getResult();
            $result = array_values(array_filter($svtResult ?? [], fn($svt) => empty($svt->getFilters()) || is_null($svt->getFilters())));
        }
        return $result;
    }
}
